import { LoadingService, Menu } from '@zeedhi/common';
import { I18n, Config, Metadata } from '@zeedhi/core';
import TekLibAuthApi from './TekLibAuthApi';
import TekLibAuthService from './TekLibAuthService';
import { Util } from '../Utilities/Util';
import { TekLibLocalStorage } from '../Helpers/TekLibLocalStorage';

export class TekLibAuth {
  private static tekLibAuthService: TekLibAuthService = TekLibAuthService.getInstance();

  public static async startSession(userData: any) {
    LoadingService.show();
    await this.tekLibAuthService.startSession(userData);
    LoadingService.hide();
  }

  public static showMessageExpiredMenu() {
    this.tekLibAuthService.showMessageExpiredMenu();
  }

  public static async logout() {
    await this.tekLibAuthService.logout();
  }

  public static async logoutConfirm() {
    await this.tekLibAuthService.logoutConfirm();
  }

  public static async updateSession(user: object, accessProfile: object[], otherSessionValues: object) {
    return new Promise((resolve, reject) => {
      TekLibAuthApi.updateSession(user, accessProfile, otherSessionValues)
        .then((response) => {
          if (!response.data.user) {
            reject(response);
          }
          localStorage.setItem('LOGIN_USER', response.data.user);
          resolve(true);
        })
        .catch((error) => {
          reject(error);
        });
    });
  }

  public static saveCollectedData(collectionIds: any, operationId: any, productId = null,
    operatorCode = null, operatorName = null, organizationId = null): Promise<any> {
    return TekLibAuthApi.saveCollectedData(collectionIds, operationId, productId, operatorCode, operatorName, organizationId);
  }

  public static async startLib() {
    window.addEventListener('message', this.handleMessageEvent);
    if (!TekLibLocalStorage.getItem('IS_NEXT')) {
      this.tekLibAuthService.sendVersionInfoForZeedhiAngular();
    }
    await this.tekLibAuthService.checkExpiredSession();
    if (Util.isProduct()) {
      TekLibAuthService.getInstance().setInitLocalStorage();
    }
  }

  public static async setInitialData(route: string, loginRoutePath: string) {
    this.updateToken();
    this.updateLanguage();
    this.updateSeparator();

    const notLoginRoute = route !== loginRoutePath;
    if (notLoginRoute
        && (route === '/' && localStorage.getItem('LOGIN_TOKEN') && !localStorage.getItem('LOGIN_USER'))
    ) {
      this.updateHash();
      this.updateKeepConnected();

      const userData = {
        loginUser: null,
        loginToken: localStorage.getItem('LOGIN_TOKEN'),
        loginLanguage: localStorage.getItem('LOGIN_LANGUAGE'),
        loginKeepConnected: localStorage.getItem('LOGIN_KEEP_CONNECTED'),
        loginHash: localStorage.getItem('LOGIN_HASH'),
        notRedirect: true,
      };
      await this.tekLibAuthService.startSession(userData);
      try {
        const menu = Metadata.getInstance<Menu>('main-menu');
        menu.items = JSON.parse(<string>localStorage.getItem('MENU')) || [];
      } catch (e) {}
    }
  }

  private static updateToken() {
    const authToken = Util.getUrlParameter('authToken');
    if (authToken) {
      localStorage.setItem('LOGIN_TOKEN', authToken);
    }
  }

  private static updateHash() {
    const hash = Util.getUrlParameter('hash');
    if (hash) {
      localStorage.setItem('LOGIN_HASH', hash);
    }
  }

  private static updateKeepConnected() {
    const keepConn = Util.getUrlParameter('keepConn');
    if (keepConn) {
      localStorage.setItem('LOGIN_KEEP_CONNECTED', keepConn);
    }
  }

  private static updateLanguage() {
    const language = Util.getUrlParameter('lang') || localStorage.getItem('LOGIN_LANGUAGE');
    if (language) {
      const languageParts = language.split('_');
      const lang = `${languageParts[0]}-${languageParts[1].toUpperCase()}`;
      localStorage.setItem('LOGIN_LANGUAGE', language);
      I18n.changeLanguage(lang);
      Util.changeDayjsLocale(lang);
    }
  }

  private static updateSeparator() {
    const separator = Util.getUrlParameter('sep') || localStorage.getItem('LOGIN_SEPARATOR');
    if (separator) {
      localStorage.setItem('LOGIN_SEPARATOR', separator);
      if (Config.masks.currencyMask) {
        const cMask: any = Config.masks.currencyMask;
        cMask.digitGroupSeparator = separator[0];
        cMask.decimalCharacter = separator[1];
        Config.masks.currencyMask = cMask;
      }
      if (Config.masks.numberMask) {
        const nMask: any = Config.masks.numberMask;
        nMask.digitGroupSeparator = separator[0];
        nMask.decimalCharacter = separator[1];
        Config.masks.numberMask = nMask;
      }
    }
  }

  private static async handleMessageEvent(event: any) {
    if (event.data.type === 'createSessionTime') {
      TekLibAuthService.getInstance().factoryExpiredSessionMethod();
    } else if (event.data.type === 'updateTranslation') {
      let isError = false;
      let message = '';
      try {
        await TekLibAuthApi.updateTranslation();
      } catch (error) {
        isError = true;
        message = error.response.data.error ? error.response.data.error : error.response.data;
      }
      window.parent.postMessage({
        type: 'updateTranslationNext',
        data: {
          isError,
          message,
        },
      }, window.location.ancestorOrigins[0]);
    } else if (event.data.type === 'keymap_iframe_keydown') {
      document.dispatchEvent(new KeyboardEvent('keydown', {
        key: event.data.message.key,
        code: event.data.message.code,
        repeat: event.data.message.repeat,
        altKey: event.data.message.altKey,
        metaKey: event.data.message.metaKey,
        ctrlKey: event.data.message.ctrlKey,
        shiftKey: event.data.message.shiftKey,
      }));
    }
  }
}
